﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local MrtWoo = MrtWoo;
local AntiSpam = MrtWoo:NewModule("AntiSpam", "AceEvent-3.0");
AntiSpam.Version = "v0.1-r14, api 4.0.0";

local opts = {
	as_options = {
		type="group",
		name="AntiSpam",
		args={
			debug = {
				type = "toggle",
				name = "Отладка",
				get = function() return MrtWoo.db.profile.modules.as_debug end,
				set = function() MrtWoo.db.profile.modules.as_debug = not MrtWoo.db.profile.modules.as_debug end,
				order=1,
			},
			
			spam_rate = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Максимальный рейтинг сообщения",
				get = function() return MrtWoo.db.profile.modules.sb_spamrate end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_spamrate = input end,
				order=2,
			},
			
			gc_wait_time = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Интервал сборки мусора",
				get = function() return MrtWoo.db.profile.modules.sb_gc_wait_time end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_gc_wait_time = input end,
				order=3,
			},

			gc_status_lifetime = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Время жизни статуса сообшения в кеше",
				get = function() return MrtWoo.db.profile.modules.sb_gc_status_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_gc_status_lifetime = input end,
				order=4,
			},
				
			gc_userinfo_lifetime = {
				type = "range",
				max = 3600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Время жизни информации о пользователе в кеше",
				get = function() return MrtWoo.db.profile.modules.sb_gc_userinfo_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_gc_userinfo_lifetime = input end,
				order=5,
			},	
		}
	}
};

function AntiSpam:SetDefaults()
	MrtWoo.defaults.profile.modules.as_debug = false;
	MrtWoo.defaults.profile.modules.sb_spamrate = 100;
	MrtWoo.defaults.profile.modules.sb_gc_wait_time = 150;
	MrtWoo.defaults.profile.modules.sb_gc_status_lifetime = 60;
	MrtWoo.defaults.profile.modules.sb_gc_userinfo_lifetime = 600;
end;

function AntiSpam:OnInitialize()
	self.SetDefaults();
	
	self.Monitor = MrtWoo_AntiSpamMonitor:new();
	
	table.insert(MrtWoo.options.plugins, opts);
end;

function AntiSpam:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;

function AntiSpam:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;

function AntiSpam:MessagesEventFilter(event, ...)
	local msg  = select(1, ...);
	
	if not msg then
		return false, ...;
	end;
		
	local msgId = select(11, ...);

	local msgStatus = AntiSpam.Monitor:GetMsgStatus(msgId);
	
	if msgStatus == nil then
		
		local author = select(2, ...);

		if MrtWoo.MyName ~= author then
			msgStatus = AntiSpam.Monitor:CheckMessage(author, msg);
		else
			msgStatus = false;
		end;
		
		AntiSpam.Monitor:SetMsgStatus(msgId, msgStatus);
		
	end;

	return msgStatus, ...;
	
end;
