--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

MrtWoo_AntiSpamMonitor = {};

function MrtWoo_AntiSpamMonitor:new(o)
	o = o or {};
	setmetatable(o, self);
	self.__index = self;
	self.users = {};
	self.statuses = {};
	self.BlockList = MrtWoo_AntiSpamBlockList:new();
	return o;
end

function MrtWoo_AntiSpamMonitor:CheckMessage(username, message)
	
	self:SmartClean();
	
	local lwrName = string.lower(username);
	
	local res = self.users[lwrName];
	
	local v = {};
	v.time = GetTime();	
	v.message = message;
	v.is_spam = false;
	v.is_flud = false;
	
	local rate = 0;

	if not res then
		rate = self.BlockList:RateTradeMessage(message);		
	else
		if not res.is_spam then
			if (res.message ~= message) then
				rate = self.BlockList:RateTradeMessage(message);
			else
				v.is_flud = true;
			end;
		else
			v.is_spam = true;
		end;
	end;
	
	if (rate >= MrtWoo.db.profile.modules.sb_spamrate) then
		v.is_spam = true;
		MrtWoo:Print('Сообщения от игрока "'..username..'" заблокированы, причина: спам! Рейтинг сообщения: ', rate);
	end;
	
	if MrtWoo.db.profile.modules.as_debug and (v.is_spam or v.is_flud) then
		MrtWoo:Print('!OK', username, message);
	end;
	
	self.users[lwrName] = v;
	return v.is_spam or v.is_flud;
end;

function MrtWoo_AntiSpamMonitor:SetMsgStatus(msgId, status)
	local v = {};

	v.time = GetTime();	
	v.status = status;
	
	self.statuses[msgId] = v;

	-- MrtWoo:Print(msgId, v.time, v.status);
	-- table.foreach(self.statuses, MrtWoo.Print);
end;

function MrtWoo_AntiSpamMonitor:GetMsgStatus(msgId)
	if self.statuses[msgId] == nil then
		return nil;
	end;
	return self.statuses[msgId].status;
end;

MrtWoo_AntiSpamMonitor.LastCleanTime = 0;

function MrtWoo_AntiSpamMonitor:SmartClean()
	local c_time = GetTime();
	if c_time - self.LastCleanTime > MrtWoo.db.profile.modules.sb_gc_wait_time then 
		table.foreach(self.statuses, 
			function(k, v) 
				if c_time - v.time > MrtWoo.db.profile.modules.sb_gc_status_lifetime then 
					if MrtWoo.db.profile.modules.as_debug then
						MrtWoo:Print('S REMOVE', k, c_time - v.time);
					end;
					self.statuses[k] = nil;
				end;
			end
		);
		
		table.foreach(self.users, 
			function(k, v) 
				if c_time - v.time > MrtWoo.db.profile.modules.sb_gc_userinfo_lifetime then 
					if MrtWoo.db.profile.modules.as_debug then
						MrtWoo:Print('U REMOVE', k, c_time - v.time);
					end;
					self.users[k] = nil;
				end;
			end
		);		
		
		-- table.foreach(self.statuses, MrtWoo.Print);
		
		self.LastCleanTime = c_time;
	end;
end;